import { UserId } from '../../user/types';
import { z } from 'zod';
import { userBooksParamsParser, userBooksQueryParser, usersBooksQueryParser } from './parsers';
import { BookId, BookType, JournalEntryId, ShelfId } from '../types';
import { PaymentProvider } from '../../paymentProvider/types';

export type UsersBooksQuery = z.infer<typeof usersBooksQueryParser>;
export type UserBooksQuery = z.infer<typeof userBooksQueryParser>;
export type UserBooksParams = z.infer<typeof userBooksParamsParser>;

export interface DbBookRow<T extends BookType = BookType> extends BookSums {
  id: BookId;
  shelfId: ShelfId;
  type: T;
}

export interface DbBookEntryRow {
  journalEntryId: JournalEntryId;
  bookId: BookId;
  toAmount: bigint;
  fromAmount: bigint;
  bookSumTo: bigint;
  bookSumFrom: bigint;
}

interface BookSums {
  sumFrom: bigint;
  sumTo: bigint;
}

export interface UsersBooksReportRow {
  userId: UserId;
  provider: PaymentProvider;
  books: {
    [t in BookType]: BookSums;
  };
}

export interface UsersBooksReportResponse {
  rows: readonly UsersBooksReportRow[];
  totals: UsersBooksReportRow['books'];
  count: number;
}

export interface UserBooksReportResponse {
  books: readonly DbBookRow[];
  bookEntries: readonly DbBookEntryRow[];
}

type BookToJson<T extends { sumTo: bigint; sumFrom: bigint }> = Omit<T, 'sumTo' | 'sumFrom'> & {
  sumTo: string;
  sumFrom: string;
};

type BooksToJson<
  T extends {
    [k in BookType]: B;
  },
  B extends { sumTo: bigint; sumFrom: bigint } = T[BookType]
> = {
  [k in BookType]: BookToJson<T[k]>;
};

type BookEntryToJson<T extends { bookSumTo: bigint; bookSumFrom: bigint }> = Omit<T, 'bookSumTo' | 'bookSumFrom'> & {
  bookSumTo: string;
  bookSumFrom: string;
};

// bigints to json, TODO a little bit problematic
export interface UsersBooksReportResponseSerialized {
  rows: (Omit<UsersBooksReportResponse['rows'][number], 'books'> & {
    books: BooksToJson<UsersBooksReportResponse['rows'][number]['books']>;
  })[];
  totals: BooksToJson<UsersBooksReportResponse['totals']>;
  count: UsersBooksReportResponse['count'];
}

// bigints to json, TODO a little bit problematic
export interface UserBooksReportResponseSerialized {
  books: BookToJson<UserBooksReportResponse['books'][number]>[];
  bookEntries: BookEntryToJson<UserBooksReportResponse['bookEntries'][number]>[];
}
