"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.channelTypeParser = exports.collectiveChannelCreateInput = exports.blogIdTraitParser = exports.channelGetOneParamsParser = exports.channelCompositeIdParser = exports.channelIdParser = exports.channelListParamsParser = void 0;
const zod_1 = require("zod");
const queryParams_1 = require("../common/parsers/queryParams");
const parsers_1 = require("../common/parsers");
const parsers_2 = require("../stripe/parsers");
const parsers_id_1 = require("../blog/parsers.id");
const parsers_id_2 = require("../collective/parsers.id");
const parsers_admin_1 = require("../collective/parsers.admin");
const constants_1 = require("./constants");
const parsers_id_3 = require("../mixspace/parsers.id");
exports.channelListParamsParser = queryParams_1.firestoreListParamsParser
    .merge(zod_1.z
    .object({
    adminId: parsers_admin_1.firebaseAdminIdParser,
})
    .partial())
    .strict();
exports.channelIdParser = parsers_id_1.firebaseBlogIdParser.or(parsers_id_2.firebaseCollectiveIdParser).or(parsers_id_3.mixspaceIdParser);
// asserted in types
exports.channelCompositeIdParser = zod_1.z.discriminatedUnion('type', [
    zod_1.z.object({
        type: zod_1.z.literal(constants_1.BLOG),
        id: parsers_id_1.firebaseBlogIdParser,
    }),
    zod_1.z.object({
        type: zod_1.z.literal(constants_1.COLLECTIVE),
        id: parsers_id_2.firebaseCollectiveIdParser,
    }),
    zod_1.z.object({
        type: zod_1.z.literal(constants_1.MIXSPACE),
        id: parsers_id_3.mixspaceIdParser,
    }),
]);
exports.channelGetOneParamsParser = zod_1.z.object({
    id: exports.channelIdParser,
});
exports.blogIdTraitParser = zod_1.z.object({
    id: parsers_id_1.firebaseBlogIdParser,
});
exports.collectiveChannelCreateInput = zod_1.z.object({
    MV: parsers_1.nullableStringParser,
    admins: zod_1.z.array(parsers_id_1.firebaseBlogIdParser),
    alias: parsers_1.nullableStringParser,
    blogs: zod_1.z.array(parsers_id_1.firebaseBlogIdParser),
    collectiveSupport: zod_1.z.boolean().default(false),
    common: zod_1.z.boolean().default(false),
    commonBlogs: zod_1.z.array(parsers_id_1.firebaseBlogIdParser),
    description: parsers_1.nullableStringParser,
    gofa: parsers_1.nullableBoolParser,
    hq: parsers_1.nullableStringParser,
    image: parsers_1.nullableUrlStringParser,
    launched: zod_1.z.boolean().default(false),
    location: parsers_1.nullableStringParser,
    mv: parsers_1.nullableStringParser,
    name: zod_1.z.string(),
    //Need to find where performances are stored (collection)
    // performanceIDs: z.array(collectiveChannelCreatePerformancesValidation).default([]),
    planetFlyer: parsers_1.nullableStringParser,
    planetImage: parsers_1.nullableUrlStringParser,
    planetName: parsers_1.nullableStringParser,
    planetPosition: zod_1.z.object({
        x: parsers_1.nullableNumberParser,
        y: parsers_1.nullableNumberParser,
    }),
    price: parsers_1.nullableNumberParser,
    radio: parsers_1.nullableBoolParser,
    smallImage: parsers_1.nullableUrlStringParser,
    sourceCollective: exports.channelIdParser,
    //Not validating sourceCollectiveData, which is just the source collective nested in this collective, probably for common collectives
    stripeProductID: parsers_2.stripeProductIdParser,
    subscribers: zod_1.z.array(exports.channelIdParser).default([]),
    type: parsers_1.nullableStringParser,
    updatedBy: parsers_1.nullableStringParser,
    verified: zod_1.z.boolean().default(false),
});
exports.channelTypeParser = zod_1.z.enum(constants_1.CHANNEL_TYPES);
