"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.payinAmountUnitParser = exports.payinApiAmountParser = exports.payinUnitParser = exports.payinAmountParser = exports.payinListQueryParser = exports.payinApiSrcDstCompositeParser = exports.payinApiSrcDstCompositeParserOptional = exports.payinApiDstCompositeParser = exports.payinApiSrcCompositeParser = exports.SEPARATOR = void 0;
const zod_1 = __importDefault(require("zod"));
const types_1 = require("../utils/types");
const constants_1 = require("./constants");
const queryParams_1 = require("../common/parsers/queryParams");
const parsers_1 = require("../currencies/parsers");
exports.SEPARATOR = '::'; // just coincides with UUID_SEPARATOR
// "api src" - looks just like a normal graph src but is not graph src; abstractions are just similar
// (in truth, its because the older api already works like that but we'd like to both move from the graph terminology and keep compatibility)
// [8c2eab11-1e88-41dc-a520-ed22af399f96]
const makeApiIdParser = (literals) => {
    const makeType = (literal) => zod_1.default.object({
        type: zod_1.default.literal(literal),
        id: zod_1.default.string().transform((0, types_1.cast)()),
    });
    // to procure type
    const first = makeType(literals[0]);
    return zod_1.default.preprocess((uuid) => {
        // no throws in preprocess https://github.com/colinhacks/zod/issues/1460
        if (typeof uuid !== 'string') {
            console.error(`Expected string, got ${typeof uuid}: ${uuid}`);
            return uuid;
        }
        const [type, id] = uuid.split(exports.SEPARATOR);
        return { type, id: id };
    }, 
    // trick zod https://github.com/colinhacks/zod/issues/1442
    zod_1.default.discriminatedUnion('type', [first, ...literals.slice(1).map(makeType)]));
};
const srcLiterals = [constants_1.PAYIN_USER_API_LITERAL_TYPE];
exports.payinApiSrcCompositeParser = makeApiIdParser(srcLiterals);
const dstLiterals = [constants_1.PAYIN_CHANNEL_API_LITERAL_TYPE];
exports.payinApiDstCompositeParser = makeApiIdParser(dstLiterals);
exports.payinApiSrcDstCompositeParserOptional = zod_1.default.object({
    src: exports.payinApiSrcCompositeParser.optional(),
    dst: exports.payinApiDstCompositeParser.optional(),
});
exports.payinApiSrcDstCompositeParser = zod_1.default.object({
    src: exports.payinApiSrcCompositeParser,
    dst: exports.payinApiDstCompositeParser,
});
exports.payinListQueryParser = queryParams_1.queryOffsetLimitParser.merge(exports.payinApiSrcDstCompositeParserOptional);
exports.payinAmountParser = parsers_1.currencyAmountParser;
exports.payinUnitParser = parsers_1.currencyParser;
exports.payinApiAmountParser = zod_1.default.object({
    amt: exports.payinAmountParser,
});
exports.payinAmountUnitParser = zod_1.default
    .object({
    unit: exports.payinUnitParser,
})
    .merge(exports.payinApiAmountParser);
