import z from 'zod';
import { SUBSCRIPTION_STATUSES } from './constants';
import { StripeSubscriptionId } from '../stripe/types';
import { Currency } from '../currencies/types';
import { subscriptionCreateInputParser, subscriptionCreateResponseParser, subscriptionDeleteParamsParser, subscriptionListQueryParser, subscriptionListResponseParser, subscriptionUpdateInputParser, subscriptionUpdateParamsParser, subscriptionUpdateResponseParser } from './parsers';
import { PAYIN_ORIGIN_TYPES, PAYIN_PROVIDERS } from '../payin/constants';
import { ApiPayinResultMixin, PayinDst, PayinSrc } from '../payin/types';
export declare type SubscriptionId = StripeSubscriptionId;
export declare type SubscriptionProvider = typeof PAYIN_PROVIDERS[number];
export declare type SubscriptionStatus = typeof SUBSCRIPTION_STATUSES[number];
export declare type SubscriptionOriginType = typeof PAYIN_ORIGIN_TYPES[number];
declare type SubscriptionOriginIdPerTypeMapping = {
    paypal: never;
    stripe: StripeSubscriptionId;
};
export declare type SubscriptionOriginIdForType<T extends SubscriptionOriginType> = SubscriptionOriginIdPerTypeMapping[T];
export declare type ApiSubscription = {
    amt: number;
    unit: Currency;
    src: PayinSrc;
    dst: PayinDst;
} & ApiPayinResultMixin<'subscription'>;
export declare type SubscriptionListApiQuery = z.infer<typeof subscriptionListQueryParser>;
export declare type SubscriptionListApiResponse = z.infer<typeof subscriptionListResponseParser>;
export declare type SubscriptionUpdateApiParams = z.infer<typeof subscriptionUpdateParamsParser>;
export declare type SubscriptionUpdateApiBody = z.infer<typeof subscriptionUpdateInputParser>;
export declare type SubscriptionUpdateApiResponse = z.infer<typeof subscriptionUpdateResponseParser>;
export declare type SubscriptionCreateApiBody = z.infer<typeof subscriptionCreateInputParser>;
export declare type SubscriptionCreateApiResponse = z.infer<typeof subscriptionCreateResponseParser>;
export declare type SubscriptionDeleteApiParams = z.infer<typeof subscriptionDeleteParamsParser>;
export {};
