import z from 'zod';
import { tipCreateInputParser, tipCreateResponseParser, tipListQueryParser, tipListResponseParser } from './parsers';
import { payinApiDstCompositeParser, payinApiSrcCompositeParser } from '../payin/parsers';
import { PAYIN_ORIGIN_TYPES, PAYIN_PROVIDERS } from '../payin/constants';
import { StripeTipId } from '../stripe/types';
import { Currency } from '../currencies/types';
import { ApiPayinResultMixin, PayinDst, PayinSrc } from '../payin/types';
export declare type TipId = StripeTipId;
export declare type TipProvider = typeof PAYIN_PROVIDERS[number];
export declare type TipOriginType = typeof PAYIN_ORIGIN_TYPES[number];
declare type TipOriginIdPerTypeMapping = {
    paypal: never;
    stripe: StripeTipId;
};
export declare type TipOriginIdForType<T extends TipOriginType> = TipOriginIdPerTypeMapping[T];
declare const TIP_STATUSES: readonly ["INTENT", "PROCESSING", "CANCELED", "SUCCEEDED", "REQUIRES_ACTION", "REQUIRES_CAPTURE", "REQUIRES_CONFIRMATION", "REQUIRES_PAYMENT_METHOD"];
export declare type TipStatus = typeof TIP_STATUSES[number];
export declare type ApiTip = {
    amt: number;
    unit: Currency;
    src: PayinSrc;
    dst: PayinDst;
} & ApiPayinResultMixin<'tip'>;
export declare type ApiTipSrcComposite = z.infer<typeof payinApiSrcCompositeParser>;
export declare type ApiTipDstComposite = z.infer<typeof payinApiDstCompositeParser>;
export declare type TipListApiQuery = z.infer<typeof tipListQueryParser>;
export declare type TipListApiResponse = z.infer<typeof tipListResponseParser>;
export declare type TipCreateApiBody = z.infer<typeof tipCreateInputParser>;
export declare type TipCreateApiResponse = z.infer<typeof tipCreateResponseParser>;
export {};
