import { z } from 'zod';
import { PAYMENT_METHOD_ORIGIN_ID_PARAM_NAME, PAYMENT_METHOD_ORIGIN_TYPE_PARAM_NAME } from './routes.constant';
import { stripePaymentMethodIdParser } from '../stripe/parsers';
import { makeLimitParser } from '../common/parsers/queryParams';

export const stripePamentMethodOriginIdParser = z.object({
  [PAYMENT_METHOD_ORIGIN_ID_PARAM_NAME]: stripePaymentMethodIdParser,
  [PAYMENT_METHOD_ORIGIN_TYPE_PARAM_NAME]: z.literal('stripe'),
});
export const paypalPaymentMethodOriginIdParser = z.object({
  [PAYMENT_METHOD_ORIGIN_ID_PARAM_NAME]: z.string(), // TODO
  [PAYMENT_METHOD_ORIGIN_TYPE_PARAM_NAME]: z.literal('paypal'),
});

// TODO may become different for payin/payout if origins become different
export const paymentMethodCompositeKeyParamsParser = z.discriminatedUnion(PAYMENT_METHOD_ORIGIN_TYPE_PARAM_NAME, [
  stripePamentMethodOriginIdParser,
  paypalPaymentMethodOriginIdParser,
]);
export const paymentMethodLimitParser = z
  .number()
  .min(0)
  .max(100 /*stripe max but don't expose stripe vars here*/)
  .default(10 /*stripe default limit but don't expose stripe vars here*/);
export const paymentMethodListLimitParser = makeLimitParser(paymentMethodLimitParser);
