import z from 'zod';
import { PayoutBatchItemCompositeKey } from './types';
import { currencyAmountParser } from '../currencies/parsers';
import { Subset } from '../utils/types';
import { PAYOUT_BATCH_ID_PARAM_NAME } from './constants';
import { nonEmptyStringParser } from '../utils/parsers';
import { payoutBatchItemCompositeKeySerializedParser } from './parsers.serialized';
import { payoutBatchItemCompositeKeyParser } from './parsers.common';

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertPayoutBatchItemCompositeKeyParser1 = Subset<
  PayoutBatchItemCompositeKey,
  z.infer<typeof payoutBatchItemCompositeKeyParser>
>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertPayoutBatchItemCompositeKeyParser2 = Subset<
  z.infer<typeof payoutBatchItemCompositeKeyParser>,
  PayoutBatchItemCompositeKey
>;

export const payoutAmountParser = currencyAmountParser;
export const payoutParamsParser = z.object({
  [PAYOUT_BATCH_ID_PARAM_NAME]: nonEmptyStringParser,
});

export const payoutBodyParser = z.object({
  items: z.record(payoutBatchItemCompositeKeySerializedParser, payoutAmountParser),
  dry: z.boolean().optional().default(true),
});

export const payoutResultParser = z.object({
  successes: z.array(payoutBatchItemCompositeKeySerializedParser),
  failures: z.record(payoutBatchItemCompositeKeySerializedParser, z.string()),
});
