import type { ChannelId } from '../channel/types';
import { isChannelId } from '../channel/utils';
import { StripePlanId } from './types';

export const STRIPE_PLAN_SEPARATOR = '_';
export const parseStripePlanPartials = (id: StripePlanId) => {
  const [, channelIdString, amtString, productIdString] = id.split(STRIPE_PLAN_SEPARATOR);
  return {
    channelIdString,
    amtString,
    productIdString,
  };
};

// plan_khDLMhDnklRNF5G4kpBZ_1000_prod_prod_HbyxcuwcBSIuUW and such
export const getChannelIdFromPriceId = (priceId: StripePlanId): null | ChannelId => {
  const s = parseStripePlanPartials(priceId).channelIdString;
  if (!isChannelId(s)) {
    console.error(`Price ${priceId} has no channel id`);
    return null;
  }
  return s;
};
