import z from 'zod';
import { SUBSCRIPTION_STATUSES } from './constants';
import { StripeSubscriptionId } from '../stripe/types';
import { Subset } from '../utils/types';
import { Currency } from '../currencies/types';
import {
  subscriptionCreateInputParser,
  subscriptionCreateResponseParser,
  subscriptionDeleteParamsParser,
  subscriptionListQueryParser,
  subscriptionListResponseParser,
  subscriptionUpdateInputParser,
  subscriptionUpdateParamsParser,
  subscriptionUpdateResponseParser,
} from './parsers';
import { PAYIN_ORIGIN_TYPES, PAYIN_PROVIDERS } from '../payin/constants';
import { ApiPayinResultMixin, PayinDst, PayinSrc, PayinSrcDstApiQueryMixin } from '../payin/types';

export type SubscriptionId = StripeSubscriptionId;
export type SubscriptionProvider = typeof PAYIN_PROVIDERS[number];
export type SubscriptionStatus = typeof SUBSCRIPTION_STATUSES[number];
export type SubscriptionOriginType = typeof PAYIN_ORIGIN_TYPES[number];
type SubscriptionOriginIdPerTypeMapping = {
  paypal: never; // PaypalSubscriptionId,
  stripe: StripeSubscriptionId;
};
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertSubscriptionOriginIdPerTypeMapping1 = Subset<
  keyof SubscriptionOriginIdPerTypeMapping,
  SubscriptionOriginType
>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertSubscriptionOriginIdPerTypeMapping2 = Subset<
  SubscriptionOriginType,
  keyof SubscriptionOriginIdPerTypeMapping
>;
export type SubscriptionOriginIdForType<T extends SubscriptionOriginType> = SubscriptionOriginIdPerTypeMapping[T];

// amt, status, unit, maybe client_secret, maybe origin_id
// backward compat with current frontend
export type ApiSubscription = {
  amt: number;
  unit: Currency;
  src: PayinSrc;
  dst: PayinDst;
} & ApiPayinResultMixin<'subscription'>;

export type SubscriptionListApiQuery = z.infer<typeof subscriptionListQueryParser>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertSubscriptionListApiQuery = Subset<SubscriptionListApiQuery, PayinSrcDstApiQueryMixin>;
export type SubscriptionListApiResponse = z.infer<typeof subscriptionListResponseParser>; // questionable whether we want to encode channel type there; probably not
export type SubscriptionUpdateApiParams = z.infer<typeof subscriptionUpdateParamsParser>;
export type SubscriptionUpdateApiBody = z.infer<typeof subscriptionUpdateInputParser>;
export type SubscriptionUpdateApiResponse = z.infer<typeof subscriptionUpdateResponseParser>;
export type SubscriptionCreateApiBody = z.infer<typeof subscriptionCreateInputParser>;
export type SubscriptionCreateApiResponse = z.infer<typeof subscriptionCreateResponseParser>;
export type SubscriptionDeleteApiParams = z.infer<typeof subscriptionDeleteParamsParser>;
