import z from 'zod';
import { Subset } from '../utils/types';
import { tipCreateInputParser, tipCreateResponseParser, tipListQueryParser, tipListResponseParser } from './parsers';
import { payinApiDstCompositeParser, payinApiSrcCompositeParser } from '../payin/parsers';
import { PAYIN_ORIGIN_TYPES, PAYIN_PROVIDERS } from '../payin/constants';
import { StripeTipId } from '../stripe/types';
import { Currency } from '../currencies/types';
import { ApiPayinResultMixin, PayinDst, PayinSrc, PayinSrcDstApiQueryMixin } from '../payin/types';

export type TipId = StripeTipId;
export type TipProvider = typeof PAYIN_PROVIDERS[number];
export type TipOriginType = typeof PAYIN_ORIGIN_TYPES[number];
type TipOriginIdPerTypeMapping = {
  paypal: never; // PaypalTipId,
  stripe: StripeTipId;
};
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipOriginIdPerTypeMapping1 = Subset<keyof TipOriginIdPerTypeMapping, TipOriginType>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipOriginIdPerTypeMapping2 = Subset<TipOriginType, keyof TipOriginIdPerTypeMapping>;
export type TipOriginIdForType<T extends TipOriginType> = TipOriginIdPerTypeMapping[T];

// currently mirrors Stripe https://stripe.com/docs/payments/intents a lot; TODO inner statuses
const TIP_STATUSES = [
  'INTENT',
  'PROCESSING',
  'CANCELED',
  'SUCCEEDED',
  'REQUIRES_ACTION',
  'REQUIRES_CAPTURE',
  'REQUIRES_CONFIRMATION',
  'REQUIRES_PAYMENT_METHOD',
] as const;

export type TipStatus = typeof TIP_STATUSES[number];

export type ApiTip = {
  amt: number;
  unit: Currency;
  src: PayinSrc;
  dst: PayinDst;
} & ApiPayinResultMixin<'tip'>;

export type ApiTipSrcComposite = z.infer<typeof payinApiSrcCompositeParser>;
export type ApiTipDstComposite = z.infer<typeof payinApiDstCompositeParser>;

export type TipListApiQuery = z.infer<typeof tipListQueryParser>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipListApiQuery = Subset<TipListApiQuery, PayinSrcDstApiQueryMixin>;
export type TipListApiResponse = z.infer<typeof tipListResponseParser>; // questionable whether we want to encode channel type there; probably not
// export type TipUpdateApiParams = z.infer<typeof tipUpdateParamsParser>;
// export type TipUpdateApiBody = z.infer<typeof tipUpdateInputParser>;
// export type TipUpdateApiResponse = z.infer<typeof tipUpdateResponseParser>;
export type TipCreateApiBody = z.infer<typeof tipCreateInputParser>;
export type TipCreateApiResponse = z.infer<typeof tipCreateResponseParser>;
// export type TipDeleteApiParams = z.infer<typeof tipDeleteParamsParser>;
