type Of<T, L extends number> = L extends 0
  ? []
  : L extends 1
  ? [T]
  : L extends 2
  ? [T, T]
  : L extends 3
  ? [T, T, T]
  : L extends 4
  ? [T, T, T, T]
  : L extends 5
  ? [T, T, T, T, T]
  : L extends 6
  ? [T, T, T, T, T, T]
  : L extends 7
  ? [T, T, T, T, T, T, T]
  : never; // welp

export const assertExactLength = <L extends number>(length: L, e0?: string) => {
  if (length < 0) throw new Error('length must be positive');
  return <B>(a: B[], e?: string): Of<B, L> => {
    if (a.length !== length) throw new Error(e0 || e || `length must be ${length}`);
    return a as unknown as Of<B, L>;
  };
};
export const getOnlyOne = <T>(a: T[], e?: string): T => {
  if (a.length !== 1) throw new Error(e || `getOnlyOne expects 1 element, got ${a.length}`);
  return a[0];
};

export const getOnlyOneC =
  (e?: string) =>
  <T>(a: T[]): T =>
    getOnlyOne(a, e);

// from "just", MIT
/*
    partition([1, 5, 2, 4, 3], n => n > 3); // [[5, 4],[1, 2, 3]]
    partition(['a', 1, 2, 'b'], x => typeof x == 'string'); // [['a', '3'],[2, 3]]
    partition([1, 2, 3, 4], x => typeof x == 'number'); // [[1, 2, 3, 4],[]]
    partition([1, 2, 3, 4], x => typeof x == 'string'); // [[], [1, 2, 3, 4]]
    partition([], n => n > 3); // [[], []]
    partition({a: 1, b: 2}, n => n > 1); // throws
    partition(null, n => n > 1); // throws
    partition(undefined, n => n > 1); // throws
*/
export const partition = <T>(arr: T[], fn: (v: T) => boolean): [T[], T[]] => {
  if (!Array.isArray(arr)) {
    throw new Error('expected first argument to be an array');
  }
  if (typeof fn != 'function') {
    throw new Error('expected second argument to be a function');
  }
  const first = [];
  const second = [];
  const length = arr.length;
  for (let i = 0; i < length; i++) {
    const nextValue = arr[i];
    if (fn(nextValue)) {
      first.push(nextValue);
    } else {
      second.push(nextValue);
    }
  }
  return [first, second];
};
